@@ -12,6 +12,7 @@
 #include <LayerShellQt/Window>
 
 #include <KWindowSystem>
+#include <KX11Extras>
 
 #include <QCursor>
 #include <QGuiApplication>
@@ -30,6 +31,7 @@ Osd::Osd(const KScreen::OutputPtr &output, QObject *parent)
 {
     connect(output.data(), &KScreen::Output::isConnectedChanged, this, &Osd::onOutputAvailabilityChanged);
     connect(output.data(), &KScreen::Output::isEnabledChanged, this, &Osd::onOutputAvailabilityChanged);
+    m_engine.setProperty("_kirigamiTheme", QStringLiteral("KirigamiPlasmaStyle"));
 }
 
 Osd::~Osd()
@@ -55,19 +57,24 @@ void Osd::showActionSelector()
         connect(rootObject, SIGNAL(clicked(int)), this, SLOT(onOsdActionSelected(int)));
     }
 
+    auto screen = qGuiApp->screenAt(m_output->pos());
+    if (!screen) {
+        screen = qGuiApp->primaryScreen();
+    }
+
     if (KWindowSystem::isPlatformWayland()) {
         auto layerWindow = LayerShellQt::Window::get(m_osdActionSelector.get());
         layerWindow->setScope(QStringLiteral("on-screen-display"));
         layerWindow->setLayer(LayerShellQt::Window::LayerOverlay);
         layerWindow->setAnchors({});
-        layerWindow->setDesiredOutput(qGuiApp->screenAt(m_output->pos()));
+        layerWindow->setKeyboardInteractivity(LayerShellQt::Window::KeyboardInteractivityOnDemand);
+        m_osdActionSelector->setScreen(screen);
     } else {
         auto newGeometry = m_osdActionSelector->geometry();
-        auto screen = qGuiApp->screenAt(m_output->pos());
-        newGeometry.moveCenter(screen ? screen->geometry().center() : qGuiApp->primaryScreen()->geometry().center());
+        newGeometry.moveCenter(screen->geometry().center());
         m_osdActionSelector->setGeometry(newGeometry);
-        KWindowSystem::setState(m_osdActionSelector->winId(), NET::SkipPager | NET::SkipSwitcher | NET::SkipTaskbar);
-        KWindowSystem::setType(m_osdActionSelector->winId(), NET::OnScreenDisplay);
+        KX11Extras::setState(m_osdActionSelector->winId(), NET::SkipPager | NET::SkipSwitcher | NET::SkipTaskbar);
+        KX11Extras::setType(m_osdActionSelector->winId(), NET::OnScreenDisplay);
         m_osdActionSelector->requestActivate();
     }
     m_osdActionSelector->setVisible(true);
@@ -92,3 +99,5 @@ void Osd::hideOsd()
         m_osdActionSelector->setVisible(false);
     }
 }
+
+#include "moc_osd.cpp"
